﻿/*
 *      Studente: Andrea ARTUSO
 *      Classe: 4AIN
 *      A.S: 2022/2023
 *      
 *      Programma per il calcolo del Minimum Spanning Tree mediante l'algoritmo di Dijkstra a
 *      partire da un file contenente la matrice delle adiacenze.
 *      
 *      Ultima modifica: 30/04/2024
 */

using Microsoft.Win32;
using Svg;
using System;
using System.Drawing.Imaging;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Threading.Tasks;

namespace AlgoritmoDijkstraWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        string MatriceAd;
        int ImmagineCorrente = 1;
        // Endpoint quickchart per l'utilizzo delle API di graphviz
        public const string Endpoint = "https://quickchart.io/graphviz?width=800&height=800&graph=graph{";

        public MainWindow()
        {
            InitializeComponent();
        }

        // Procedura per aprire il file explorer e selezionare il file di testo contenente la matrice delle adiacenze
        private void btn_open_file_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*";

            try
            {
                if (fileDialog.ShowDialog() == true)
                    MatriceAd = File.ReadAllText(fileDialog.FileName);
                else
                {
                    return;
                }
            } catch (IOException ioe)
            {
                lbl_errore.Content = ioe.Message;
                return;
            }

            GeneraMatrice();
        }

        // Procedura per l'acquisizione della matrice dal file di testo
        private void GeneraMatrice()
        {
            string[] Righe = MatriceAd.Split('\n');

            int n = Righe[0].Split(' ').Length;

            if (Righe.Length != n)
            {
                lbl_errore.Content = "Errore: la matrice non è quadrata!";
                return;
            }

            int[,] Matrice = new int[n,n];

            for (int i=0; i<n; i++)
            {
                string[] RigaMat = Righe[i].Split(' ');
                for (int j=0; j<n; j++)
                {
                    Matrice[i, j] = Convert.ToInt32(RigaMat[j]) == -1 ? int.MaxValue : Convert.ToInt32(RigaMat[j]);
                }
            }

            // Vettore di nodi
            Dijkstra.Nodo[] grafo = new Dijkstra.Nodo[n];

            grafo[0] = new Dijkstra.Nodo(0, 0, null);
            for (int i = 1; i < n; i++)
                grafo[i] = new Dijkstra.Nodo(i, int.MaxValue, null);

            // Calcolo distanze minime
            Dijkstra.Risolvi(Matrice, grafo, n);

            string graphvizCode = "";
            // Generazione codice graphviz per l'MST (Minimum Spanning Tree)
            for (int i = 0; i < n; i++)
            {
                for (int j = 0; j < n; j++)
                    if (grafo[j].prec != null && grafo[j].prec == i)
                        graphvizCode += String.Format("{0}--{1} [taillabel={2}] ", i, j, grafo[j].costo);
            }
            graphvizCode += "}";

            string URL = Endpoint + graphvizCode;

            lbl_errore.Content = "";
            Task t1 = Task.Factory.StartNew(() => ScaricaImmagine(URL));
        }

        // Procedura per il download e la visualizzazione dell'immagine
        private void ScaricaImmagine(string URL)
        {
            int OptimalWidth = 800;
            (int, int) DimensioniImg;
            string PathnameSVG = "tmp.svg";
            string PathnamePNG = String.Format("../../../../output-grafi/grafo{0}-{1}.png", ImmagineCorrente, DateTimeOffset.Now.ToUnixTimeSeconds());      // aggiungo al nome del file il timestamp corrente

            try
            {
                Dispatcher.Invoke(() => img.Source = null);

                using (WebClient c = new WebClient())
                {
                    c.DownloadFile(URL, PathnameSVG);
                }

                var svgDocument = SvgDocument.Open(PathnameSVG);
                using (var smallBitmap = svgDocument.Draw())
                {
                    var width = smallBitmap.Width;
                    var height = smallBitmap.Height;

                    if (width != OptimalWidth)
                    {
                        width = OptimalWidth;
                        height = OptimalWidth / smallBitmap.Width * height;
                    }

                    DimensioniImg.Item1 = width;
                    DimensioniImg.Item2 = height;

                    using (var bitmap = svgDocument.Draw(width, height))
                    {
                        bitmap.Save(PathnamePNG, ImageFormat.Png);
                    }
                }

                File.Delete(PathnameSVG);

                Dispatcher.Invoke(() =>
                {
                    img.Height = (img.Width * DimensioniImg.Item1) / DimensioniImg.Item2;
                    img.Source = new BitmapImage(new Uri(System.IO.Path.GetFullPath(PathnamePNG), UriKind.Absolute));

                    lbl_downloaded.Content = "Immagine scaricata in " + System.IO.Path.GetFullPath(PathnamePNG);
                });

                ImmagineCorrente++;
            }
            catch (Exception e)
            {
                Dispatcher.Invoke(() => lbl_errore.Content = e.Message);
            }
        }
    }
}
